;       Compile with WLA-6510 (http://www.hut.fi/~vhelin/wla.html)

.INCLUDE "vic.inc"              ; memory map defines for WLA


.DEFINE Screen          $1e00   ; for unexpanded VIC
.DEFINE ColorMap        $9600

.DEFINE LoadAddr        $1001
.DEFINE CharMem         $1400
.DEFINE MidBlock        $15d0   ; after character data (but before music)

.DEFINE rasterpos       $42     ; line for raster time meter (music call)
.DEFINE VUpos           $2a     ; top line of VU meters

.DEFINE rastercolor     $11
.DEFINE screencolor     $10
.DEFINE vuc             $0e     ; vu meter background color

.INCLUDE "fisichella.inc"       ; player defines
.DEFINE INITSPEED       $1dff   ; initial song speed

.DEFINE FILENAME        $100
.DEFINE NoteNames       $110

.DEFINE LOAD            $ffd5   ; KERNAL routines for load/save memory
.DEFINE SAVE            $ffd8
.DEFINE SETLFS          $ffba
.DEFINE SETNAM          $ffbd

.ORG $0
.SECTION "BANKHEADER"
        .dw     LoadAddr
.ENDS


.DEFINE MusicFlag       $00
.DEFINE Key             $01
.DEFINE Aux             $02

.ENUM $39
BLOCK           DB              ; zero page variables
SOUND           DB
ENV             DB
SPEED           DB

VUMeter1        DB
VUMeter2        DB
VUMeter3        DB
VUMeter4        DB

LISTY           DB
CURSX           DB
CURSY           DB
WINFLAG         DB

FLASH           DB
LINEINPUT       DB
OPT             DB
.ENDE

.ENUM $4e
HEXKEY          DB              ; variables not initialized on load/save
NOTEKEY         DB
.ENDE


.ORG LoadAddr
.SECTION "MAIN" FORCE

        .DB     $0b,$10,$d1,$07,$9e,$34,$31,$30,$39     ;"2001 sys4109"
        .DB     $00,$00,$00

        sei
modify:
        jmp     FirstRun

FirstRun:
        ldy     #$a4                    ; copy tables from $1e00
_move:
        lda     TapeBuffImage-1,y
        sta     $0333,y
        cpy     #$5e
        bcs     _holdon
        lda     NoteTable-1,y
        sta     NoteNames-1,y
_holdon:
        dey
        bne     _move

        lda     #Restart&255
        sta.w   modify+1

Restart:
        lda     #$80
        sta     $028a                   ; Repeat all keys
        sta     $0291                   ; Disable case switching

        lda     #$00                    ; Direct mode off (to prevent KERNAL
        sta     $9d                     ; messages if started from monitor)

        lda     #$fd                    ; Character memory at $1400
        sta     $9005

        jsr     InitScreen
        jsr     MusicOff

MainLoop:
        jsr     CursorOn

        ldy     BLOCK                   ; write block number
        lda     Hex,y
        sta     Screen+132+6

        ldy     SPEED                   ; write song speed
        lda     Hex,y
        sta     Screen+21

        ldy     SOUND                   ; write sound number
        lda     Hex,y
        sta     Screen+132+12

        lda     SndSpeed,y              ; get & write sndspeed (resolution)
        tay
        lda     Hex,y
        sta     Screen+132+15

        jsr     GetEnv                  ; get & write sound env number
        tay
        lda     Hex,y
        sta     Screen+132+21

        lda     #$0f                    ; rewrite all 16 lines
        sta     OPT
_ilin:
        cmp     #$04                    ; BottomLine has only 4 lines
        bcs     _notbothyet

        jsr     WriteBottomLine
        lda     OPT
_notbothyet:
        jsr     WriteLine

        dec     OPT
        lda     OPT
        bpl     _ilin


_waitkey:
        jsr     FrameRoutines
        jsr     ReadKeyboard
        beq     _waitkey

        sta     Key
        jsr     FrameRoutines

        jsr     CursorOff
        jsr     CheckCmdKey
        jsr     CheckNoteHex

        jmp     MainLoop


CheckNoteHex:
        lda     WINFLAG                 ; check active window
        beq     _editwindow

_seqwindow:
        jsr     _getseqpos

        lda     NOTEKEY
        beq     _seqclear               ; clear one sequence (space)
        lda     HEXKEY
        bmi     _nogoodblock
        cmp     #$0e
        beq     _setsongend
        cmp     #$08
        bcs     _nogoodblock

        lda     ($fb),y                 ; maintain transpose value if any
        bpl     _noseqkill
        lda     #$00
_noseqkill:
        and     #$0f
        jsr     _hinybblewr

_nogoodblock:
        rts

_setsongend:
        ldy     #$00                    ; song end mark
        lda     #$fe
        sta     ($fb),y
        iny
_seqclear:
        lda     #$ff                    ; clear seq or 2nd half of end mark
        sta     ($fb),y
        jmp     _advancecursor          ; rts there

_getseqpos:
        ldx     CURSY
        jsr     FindSeqLocation
        ldy     CURSX

        rts


_editwindow:
        ldy     CURSY

        ldx     CURSX
        beq     _blockbox
        cpx     #$01
        beq     _soundbox

        lda     ENV
        beq     _nogoodeditkey          ; env 0 is not used
        jsr     GetEnvValue

        cpx     #$02
        beq     _envboxhi

        and     #$f0                    ; edit env low nybble
        ora     HEXKEY

        jmp     _generalhexwr

_envboxhi:
        and     #$0f                    ; edit env hi nybble
        bne     _hinybblewr
        cpy     HEXKEY
        beq     _nogoodeditkey          ; don't allow deadlock loop!

_hinybblewr:
        sta     Aux
        lda     HEXKEY
        clc
        rol
        rol
        rol
        rol
        ora     Aux
        ;
_generalhexwr:
        ldx     HEXKEY                  ; last minute key validity check
        bmi     _nogoodeditkey
        sta     ($fb),y                 ; write new env low nybble
        ;
_advancecursor:
        inc     CURSY
        jsr     ScrollCheck

_nogoodeditkey:
        rts

_blockbox:
        jsr     GetBlockNote            ; edit block note

        lda     NOTEKEY
        bmi     _nogoodeditkey          ; $ff = no keyboard key
        beq     _generalboxwr           ; $00 = space/del

        dec     NOTEKEY
        beq     _nogoodeditkey

        lda     SOUND
        clc
        ror
        ror
        ror
        ror
        ora     NOTEKEY
        ;
_generalboxwr:
        sta     ($fb),y                 ; write note to block or sound
        jmp     _advancecursor

_soundbox:
        jsr     GetSoundNote            ; edit sound note

        lda     NOTEKEY
        bmi     _nogoodeditkey          ; $ff = no keyboard key
        beq     _generalboxwr           ; $00 = space/del

        dec     NOTEKEY
        lda     #$80
        ora     NOTEKEY                 ; set vibrato at 0

        jmp     _generalboxwr

EditInc:
        lda     WINFLAG
        bne     _editincseq

        lda     #$20
        sta     Aux
        ;
_incdecsoundvib:
        ldx     CURSX                   ; only in sound box
        dex
        bne     _nogoodeditkey

        ldy     CURSY
        jsr     GetSoundNote

        adc     Aux
        cmp     #$1f                    ; check for bounds
        bcc     _nogoodeditkey
        sta     ($fb),y
        rts

EditDec:
        lda     WINFLAG
        bne     _editdecseq

        lda     #$e0
        sta     Aux

        jmp     _incdecsoundvib

_loopincseq:
        jsr     _getseqpos

        ldy     #$00
        lda     ($fb),y
        cmp     #$fe
        bne     _nogoodeditkey

        iny
        lda     ($fb),y
        clc
        adc     #$04
        and     #$7c
        sta     ($fb),y

        rts

_editincseq:
        jsr     _getseqtrans
        inx
        txa
_putseqtrans:
        and     #$0f
        sta     Aux
        lda     ($fb),y
        and     #$70
        ora     Aux
        sta     ($fb),y
        rts

_editdecseq:
        jsr     _getseqtrans
        dex
        txa
        jmp     _putseqtrans

_getseqtrans:
        jsr     _getseqpos
        lda     ($fb),y
        and     #$0f
        tax
        rts

LoopInc:
        lda     WINFLAG
        bne     _loopincseq

        ldx     CURSX
        dex
        bne     _nogoodloopkey

        ldy     CURSY
        jsr     GetSoundNote
        cmp     #$1f
        bcs     _nogoodloopkey

        adc     #$01                    ; increment loop position
        and     #$0f

        sta     Aux                     ; don't allow deadlock loop!
        cpy     Aux
        beq     _nogoodloopkey

        sta     ($fb),y
_nogoodloopkey:
        rts


CheckCmdKey:
        ldy     #$ff
        sty     HEXKEY
        sty     NOTEKEY
_seekkey:
        iny

        lda     CmdKeyList,y
        beq     _nomorecmdkeys

        cmp     Key                     ; matching the pressed key?
        bne     _seekkey

_keyfound:
        lda     CmdKeyJumpL,y           ; read command jmp address
        sta     $fb
        lda     CmdKeyJumpH,y
        sta     $fc
        beq     _incdec                 ; not a jump but general inc/dec

        jmp     ($fb)                   ; execute command (rts there)

_nomorecmdkeys:
        ldy     #$1d                    ; keyboard note scan (0-29)

_seeknotekey:
        lda     NoteKey,y
        cmp     Key
        beq     _notekeyfound
        dey
        bpl     _seeknotekey

_notekeyfound:
        sty     NOTEKEY
        ;
_nomorenotekeys:
        lda     Key                     ; hex key scan (0-15)
        and     #$3f                    ; mask to match key presses with
        sta     Aux                     ; petscii hex conversion table

        ldy     #$0f

_seekhexkey:
        lda     Hex,y
        cmp     Aux
        beq     _hexkeyfound
        dey
        bpl     _seekhexkey

_hexkeyfound:
        sty     HEXKEY
        ;
_nomorehexkeys:
        rts


_incdec:
        ldy     $fb
        lda     IncDecList,y            ; get inc/dec address (zeropage)
        sta     $fb                     ; $fc is already zero
        lda     IncDecList+2,y
        sta     Aux
        lda     IncDecList+1,y          ; get inc/dec add value
        ldy     #$00

        clc
        adc     ($fb),y
        and     Aux
        sta     ($fb),y

        lda     SPEED                   ; it's possible that Spd was changed
        sta     INITSPEED
        sta     SongSpeed
        ;
ScrollCheck:
        lda     WINFLAG                 ; check active window (bit 0)
        bne     SeqScroll

        lda     CURSY                   ; required by some edit operations
        and     #$0f
        sta     CURSY

        rts

SeqScroll:
        lda     LISTY
        ldx     CURSY
        cpx     #$0f
        bne     _noscrollup
        cmp     #$00
        beq     _scrolltop
        clc
        adc     #$fc
_scrolltop:
        ldx     #$00
_noscrollup:
        cpx     #$04
        bne     _noscrolldown
        cmp     #$70
        beq     _scrollbottom
        clc
        adc     #$04
_scrollbottom:
        ldx     #$03
_noscrolldown:
        sta     LISTY
        stx     CURSY

        rts


WriteBottomLine:
        tax                             ; store line number
        ora     #$10
        jsr     Locate

        jsr     FindSeqLocation         ; gets line number from x

        ldy     #$00
        ldx     #$01
        jsr     HexOut                  ; write step counter

        ldy     #$00                    ; get voice 1 block
        lda     ($fb),y
        cmp     #$fe
        beq     SeqLoop
        ldy     #$04
        ldx     #$06
        jsr     SeqBlockOut

        ldy     #$01                    ; get voice 2 block
        lda     ($fb),y
        ldy     #$09
        ldx     #$0b
        jsr     SeqBlockOut
_lasttwovoices:
        ldy     #$02
        lda     ($fb),y                 ; get voice 3 block
        ldy     #$0e
        ldx     #$10
        jsr     SeqBlockOut

        ldy     #$03
        lda     ($fb),y                 ; get voice 4 block
        ldy     #$13
        ldx     #$15
        ;
SeqBlockOut:
        cmp     #$ff
        beq     _noblock
        jsr     HexOut

        ldy     $f7                     ; get position of high nybble
        iny
        lda     #$2b                    ; write + between nybbles
        sta     ($fd),y

        rts

_noblock:
        tya                             ; oops
        tax
        lda     #$00
        jmp     NoteOut                 ; rts there

SeqLoop:
        ldy     #$01                    ; get loop position
        lda     ($fb),y
        cmp     #$ff
        beq     _songend

        ldy     #$05                    ; write loop position
        ldx     #$06
        jsr     HexOut

        ldy     #$04                    ; write up arrow for loop
        lda     #$1e
        sta     ($fd),y

_secondvoice:
        ldx     #$09                    ; no block for voice 2
        lda     #$00
        jsr     NoteOut

        jmp     _lasttwovoices

_songend:
        ldx     #$04
        lda     #$1e
        jsr     NoteOut

        jmp     _secondvoice


InitScreen:
        ldy     #$00
_cls:
        lda     #$20
        sta     Screen,y
        sta     Screen+$100,y

        lda     #$00
        sta     ColorMap,y
;        sta     ColorMap+$100,y

        iny
        bne     _cls

        lda     #(ColorMap+7*22)&255
        sta     $fd
        lda     #(ColorMap+7*22)/256
        sta     $fe

        ldx     #$0f
_boxcolor:
        ldy     #$15
_hrowlp:
        lda     hrowmap,y
        sta     ($fd),y
        dey
        bpl     _hrowlp

        lda     $fd
        clc
        adc     #$16
        sta     $fd
        bcc     _hrownopage
        inc     $fe
_hrownopage:
        dex
        bpl     _boxcolor
        ;
ReWrite:
        ldx     #$15
_write:
        lda.w   toptext,x               ; write top & mid text
        sta.w   Screen+132,x
        lda.w   midtext,x
        sta.w   Screen,x
        lda.w   lrowmap,x               ; bottom window color map
        sta.w   ColorMap+22,x
        sta.w   ColorMap+44,x
        sta.w   ColorMap+66,x
        sta.w   ColorMap+88,x
        lda     #$00
        sta     $39,x                   ; clear zero page variables
        dex
        bpl     _write

        lda     INITSPEED
        sta     SPEED

        rts


InputFilename:
        ldx     #$05
_msgout:
        lda     LoadSaveMsg,y           ; write load/save message
        sta     $1dff,x
        iny
        dex
        bne     _msgout

        stx     LINEINPUT               ; zero

        ldy     #$10
        lda     #$20
_spaces:
        sta     $1e05,y                 ; clear space for filename
        dey
        bpl     _spaces

        jsr     MusicOff


_namewaitkey:
        jsr     GetCursorColor
        ldy     LINEINPUT
        sta     ColorMap+5,y

        ldx     #$2a
_wait3:
        cpx     $9004
        bne     _wait3

        jsr     ReadKeyboard
        beq     _namewaitkey

        tax                             ; store key code

        ldy     LINEINPUT
        lda     #$00
        sta     ColorMap+5,y            ; wipe cursor

        cpx     #$0d
        beq     _nameout
        cpx     #$03
        beq     _namequit
        cpx     #$14
        bne     _notdel

        dey                             ; backspace
        bmi     _namewaitkey

        lda     #$20                    ; can delete
        sta     Screen+5,y
        sty     LINEINPUT
        jmp     _namewaitkey

_notdel:
        cpy     #$10
        beq     _namewaitkey

        cpx     #$20
        beq     _namekeyok
        cpx     #$41                    ; valid key range from 'A'
        bcc     _namewaitkey
        cpx     #$5b                    ; to 'Z'
        bcs     _namewaitkey

_namekeyok:
        txa                             ; restore key code

        and     #$7f
        sta     FILENAME,y
        and     #$3f
        sta     Screen+5,y

        inc     LINEINPUT
        jmp     _namewaitkey

_nameout:
        sty     Aux                     ; store filename length
        cpy     #$00
        beq     _namequit

        ldx     #$08                    ; default drive 8

        lda     $ba                     ; use default device instead of 0-1
        cmp     #$02
        bcc     _nogood
        
        tax                             ; use current (last accessed) drive

_nogood:
        lda     #$01                    ; logical file number
        tay                             ; secondary address
        jsr     SETLFS

        lda     Aux                     ; get filename length
        ldx     #FILENAME&255           ; filename address
        ldy     #FILENAME/256
        jsr     SETNAM

        lda     Aux                     ; filename set
        cli
_namequit:
        rts

.ENDS


.ORG CharMem
.SECTION "FONTLOW" FORCE
        .INCBIN "edfontlow.bin"
.ENDS

.ORG CharMem+39*8
.SECTION "UnusedChar" FORCE

NoteKey:
        .DB     $20                             ; space
        .DB     "ZSXDCVGBHNJMQ2W3ER5T6Y7UI9O0P" ; 29 note keys

        .DB     $00,$00                         ; unused
.ENDS

.ORG CharMem+43*8
.SECTION "FONTHI" FORCE
        .INCBIN "edfonthi.bin"
.ENDS


.ORG MidBlock
.SECTION "SUBS" FORCE

ReadKeyboard:
        jsr     $ff9f                   ; keyboard scan

        lda     #$01                    ; speed up key repeat
        sta     $028b

        lda     $c6                     ; check buffer
        beq     _nokey

        ldy     $0277                   ; get key
        ldx     #$00
_movebuf:
        lda     $0278,x                 ; scroll keyboard buffer
        sta     $0277,x
        inx
        cpx     $c6
        bne     _movebuf

        dec     $c6
        tya
_nokey:
        rts


FrameRoutines:
        lda     $900e
        and     #$0f
        sta     Aux

        ldy     #$1f
_VUprepare:
        lda     $03b8,y
        and     #$f0
        ora     Aux
        sta     $03b8,y
        dey
        bpl     _VUprepare

        ldy     #$1f
        lda     $03d7           ; $03b8 + $1f

        ldx     #$2a            ; VU meter raster line
_wait1:
        cpx     $9004
        bne     _wait1

_VUcolorize:
        sta     $900e           ; 71 cycles loop

        ldx     #$0b            ; must not hit a page boundary
_raster:
        dex
        bne     _raster

        .DB     $ea             ; nop

        lda     $03b7,y
        dey
        bpl     _VUcolorize     ; ends here

        jsr     CursorOn

        lda     #rastercolor
        ldx     #rasterpos

_wait2: cpx     $9004
        bne     _wait2

        sta     $900f

        lda     MusicFlag
        beq     _nomusic

        jsr     PlayerFrame

        lda     $900e                   ; trig value for vu meters
        and     #$0f
        asl                             ; $0f -> $1e (clears carry)
        adc     #$03                    ; -> $21
        tax

        lda     V1SoundPos
        and     #$0e
        bne     _NoVU1
        stx     VUMeter1
_NoVU1:
        lda     V2SoundPos
        and     #$0e
        bne     _NoVU2
        stx     VUMeter2
_NoVU2:
        lda     V3SoundPos
        and     #$0e
        bne     _NoVU3
        stx     VUMeter3
_NoVU3:
        lda     V4SoundPos
        and     #$0e
        bne     _NoVU4
        stx     VUMeter4
_NoVU4:


_nomusic:
        lda     #screencolor
        sta     $900f

        ldx     #$03
_VUDown:
        dec     VUMeter1,x
        bpl     _VUon
        lda     #$00
        sta     VUMeter1,x
_VUon:
        dex
        bpl     _VUDown

        ldx     VUMeter1
        ldy     #$03
        jsr     DrawVUMeter
        ldx     VUMeter2
        ldy     #$08
        jsr     DrawVUMeter
        ldx     VUMeter3
        ldy     #$0d
        jsr     DrawVUMeter
        ldx     VUMeter4
        ldy     #$12
        
DrawVUMeter:
        lda     $0334,x
        sta     Screen+22,y
        lda     $0355,x
        sta     Screen+44,y
        lda     $0376,x
        sta     Screen+66,y
        lda     $0397,x
        sta     Screen+88,y

        rts


WriteLine:
        jsr     Locate                  ; line number in both OPT and a

        ldy     OPT                     ; line number from OPT
        lda     Hex,y

        ldy     #$11                    ; step count for env
        sta     ($fd),y
        ldy     #$08                    ; step count for snd
        sta     ($fd),y
        ldy     #$00                    ; step count for block
        sta     ($fd),y

        tya                             ; pieces of raster time bar
        ldy     #$07
        sta     ($fd),y                 ; between the boxes
        ldy     #$10
        sta     ($fd),y

        ldy     OPT
        jsr     GetBlockNote

        and     #$1f                    ; write note to screen
        beq     _zeronote               ; 0 = ---
        clc
        adc     #$01                    ; but 1 = actually C#0 not C-0
_zeronote:
        ldx     #$02
        jsr     NoteOut

        ldy     OPT                     ; get sound number
        lda     ($fb),y
        bne     _readblocksnd
        lda     #$20                    ; write a space if no note
        jmp     _blocksnd

_readblocksnd:
        rol
        rol
        rol
        rol
        and     #$07
        tay
        lda     Hex,y

_blocksnd:
        ldy     #$06                    ; write sound number to screen
        sta     ($fd),y

        ldy     OPT
        jsr     GetSoundNote

        ror                             ; read sound vibrato
        ror
        ror
        ror
        and     #$0e
        bne     _nosndloop

        ldy     OPT
        lda     ($fb),y
        and     #$0f
        tay
        lda     Hex,y
        ldy     #$0f                    ; write sound loop position
        sta     ($fd),y
        lda     #$1e                    ; write up arrow
        dey
        sta     ($fd),y

        lda     #$00                    ; go write note ---
        jmp     _fromsndloop

_nosndloop:
        tay

        lda     VibLabels,y
        sta     $f7
        lda     VibLabels+1,y

        ldy     #$0f                    ; write sound vibrato
        sta     ($fd),y
        lda     $f7
        dey
        sta     ($fd),y                 ; write vibrato +/-

        ldy     OPT                     ; read sound note
        lda     ($fb),y
        and     #$1f
        clc
        adc     #$01
_fromsndloop:
        ldx     #$0a                    ; write sound note
        jsr     NoteOut

_getenv:
        ldy     SOUND                   ; get envelope parameters
        lda     SndEnvTrig,y
        beq     _envzero                ; envelope zero not usable (show 00)
        ldy     OPT
        jsr     GetEnvValue

_envzero:
        ldy     #$13
        ldx     #$15
        pha
        jsr     HexOut                  ; write envelope level & time
        pla
        and     #$0f                    ; low nybble = 0 if env. loop
        bne     _notenvloop

        lda     #$1e                    ; write up arrow if loop
        jmp     _envtip

_notenvloop:
        lda     #$2f                    ; else write a slash
_envtip:
        ldy     #$14
        sta     ($fd),y

        rts


NoteOut:
        sta     $f7                     ; multiply a by 3
        asl
        clc
        adc     $f7

        clc
        adc     #NoteNames&255
        sta     $f7
        lda     #$00
        adc     #NoteNames/256
        sta     $f8

        stx     $f9                     ; x-coordinate at row ($fd)
        lda     $fd
        clc
        adc     $f9
        sta     $f9
        lda     $fe
        adc     #$00
        sta     $fa

        ldy     #$02
_note:
        lda     ($f7),y
        sta     ($f9),y
        dey
        bpl     _note

        rts

HexOut:
        sty     $f7                     ; x coord for hi/lo nybble (must be
        stx     $f8                     ; possible to split them on screen)
        tax
        and     #$0f                    ; convert low nybble
        tay
        lda     Hex,y
        pha

        txa                             ; convert high nybble
        ror
        ror
        ror
        ror
        and     #$0f
        tay
        lda     Hex,y

        ldy     $f7                     ; write high nybble
        sta     ($fd),y
        ldy     $f8
        pla                             ; write low nybble
        sta     ($fd),y

        rts


Locate:
        tay                             ; get address of row a to ($fd)

        lda     RowsL,y                 ; (topmost line is excluded)
        sta     $fd
        lda     RowsH,y
        sta     $fe

        rts


GetCursorColor:
        inc     FLASH
        lda     FLASH
        ror
        ror
        and     #$07
        tay
        lda     CursorFlash,y

        rts

CursorOn:
        jsr     SeekCursor
        sty     Aux

        jsr     GetCursorColor

        ldy     Aux
        jmp     FillCursor

CursorOff:
        jsr     SeekCursor
        lda     #$06
        ;
FillCursor:
        sta     ($fd),y
        iny
        dex
        bne     FillCursor
        rts

SeekCursor:
        lda     WINFLAG                 ; check active window (bit 0)
        bne     _seqcursor

        lda     CURSY                   ; cursor for edit box area
        jsr     Locate

        lda     CURSX
_commoncursor:
        tay
        lda     CurSizes,y
        tax
        lda     CurCols,y
        tay

        lda     $fe
        clc
        adc     #(ColorMap-Screen)/256
        sta     $fe

        rts

_seqcursor:
        lda     CURSY
        and     #$03
        ora     #$10

        jsr     Locate

        lda     CURSX
        ora     #$04

        jmp     _commoncursor


EnvUp:
        jsr     GetEnv
        adc     #$01
_envcommon:
        rol
        rol
        rol
        rol
        and     #$30
        sta     SndEnvTrig,y
        rts

EnvDown:
        jsr     GetEnv
        adc     #$ff
        jmp     _envcommon

ResoUp:
        jsr     GetReso
        adc     #$01
_resocommon:
        and     #$07                    ; useful range $0-$7
        sta     SndSpeed,y
        rts

ResoDown:
        jsr     GetReso
        adc     #$ff
        jmp     _resocommon

GetReso:
        ldy     SOUND
        lda     SndSpeed,y
        clc
        rts

GetEnv:
        ldy     SOUND                   ; get sound env number
        lda     SndEnvTrig,y
        sta     ENV
        clc
        ror
        ror
        ror
        ror
        rts

GetEnvValue:
        ora     #$80                    ; get value of env step
        sta     $fb                     ; a = env, y = step $0-$f
        lda     #EnvData/256            
        sta     $fc
        lda     ($fb),y
        rts

GetBlockNote:
        lda     #BlockData/256
        sta     $fc

        lda     BLOCK                   ; get note at step y in BLOCK
        ora     #$08                    ; block data at fixed low $80
        jmp     _getbitshift

GetSoundNote:
        lda     #SoundData/256          ; get note at step y in SOUND
        sta     $fc

        lda     SOUND
_getbitshift:
        clc
        rol
        rol
        rol
        rol
        sta     $fb

        lda     ($fb),y                 ; carry always clear on exit
        rts

FindSeqLocation:
        lda     #SeqData/256            ; calculate sequencer step location
        sta     $fc
        txa                             ; cursor line from x
        asl
        asl
        clc
        adc     LISTY
        sta     $fb

        rts

WinSwap:
        lda     WINFLAG
        eor     #$01
        sta     WINFLAG

        lda     #$00
        sta     CURSY
        sta     CURSX
        rts

PlayBlank:                              ; F2: Play with blank screen
        lda     #$a0
        sta     $9001
        jmp     playentry

MusicOn:
        jsr     screenback              ; F1: Play music
playentry:
        lda     MusicFlag
        bne     _noretrig

        lda     #$01
        sta     MusicFlag
        jsr     PlayerInit

_noretrig:
        rts

MusicOff:
        lda     #$00                    ; Run/stop: Stop music
        sta     MusicFlag
        jsr     PlayerOff
        ;
screenback:
        lda     #$26                    ; must match with VUpos
        sta     $9001

        rts


LoadMusic:
        ldy     #$00
        jsr     InputFilename
        beq     LoadSaveDone

        lda     #$00
        jsr     LOAD

        jmp     LoadSaveDone

SaveMusic:
        ldy     #$05
        jsr     InputFilename
        beq     LoadSaveDone

        lda     #$1a                    ; from $1a00 to $1e00
        sta     $02
        lda     #$00
        sta     $01
        tax
        lda     #$01
        ldy     #$1e

        jsr     SAVE
        ;
LoadSaveDone:
        sei
        jmp     ReWrite                 ; rts there

.ENDS


.SECTION "DATA" FORCE

CmdKeyList:
        .DB     $03                     ; run/stop
        .DB     $85                     ; f1
        .DB     $89                     ; f2

        .DB     $86                     ; f3
        .DB     $8a                     ; f4
        .DB     $87                     ; f5
        .DB     $8b                     ; f6
        .DB     $2f                     ; /
        .DB     $3f                     ; shift + /

        .DB     $3a                     ; :
        .DB     $5b                     ; shift + :
        .DB     $3b                     ; ;
        .DB     $5d                     ; shift + ;

        .DB     $1d                     ; cursor right
        .DB     $9d                     ; cursor left
        .DB     $11                     ; cursor down
        .DB     $91                     ; cursor up

        .DB     $88                     ; f7
        .DB     $8c                     ; f8
        .DB     $13                     ; home

        .DB     $2b                     ; +
        .DB     $2d                     ; -

        .DB     $5e                     ; arrow up
        ;                               ; 1 byte truncated
VibLabels:
        .DB     $00,$1e
        .INCBIN "vibnames.bin"


CmdKeyJumpL:
        .DB     MusicOff&255            ; run/stop
        .DB     MusicOn&255             ; f1
        .DB     PlayBlank&255           ; f2

        .DB     0                       ; f3
        .DB     3                       ; f4
        .DB     6                       ; f5
        .DB     9                       ; f6
        .DB     12                      ; /
        .DB     15                      ; shift + /

        .DB     ResoUp&255              ; :
        .DB     ResoDown&255            ; shift + :
        .DB     EnvUp&255               ; ;
        .DB     EnvDown&255             ; shift + ;

        .DB     18                      ; cursor right
        .DB     21                      ; cursor left
        .DB     24                      ; cursor down
        .DB     27                      ; cursor up

        .DB     WinSwap&255             ; f7
        .DB     SaveMusic&255           ; f8
        .DB     LoadMusic&255           ; home

        .DB     EditInc&255             ; +
        .DB     EditDec&255             ; -

        .DB     LoopInc&255             ; arrow up

CmdKeyJumpH:
        .DB     MusicOff/256            ; run/stop
        .DB     MusicOn/256             ; f1
        .DB     PlayBlank/256           ; f2

        .DB     0                       ; f3
        .DB     0                       ; f4
        .DB     0                       ; f5
        .DB     0                       ; f6
        .DB     0                       ; /
        .DB     0                       ; shift + /

        .DB     ResoUp/256              ; :
        .DB     ResoDown/256            ; shift + :
        .DB     EnvUp/256               ; ;
        .DB     EnvDown/256             ; shift + ;

        .DB     0                       ; cursor right
        .DB     0                       ; cursor left
        .DB     0                       ; cursor up
        .DB     0                       ; cursor down

        .DB     WinSwap/256             ; f7
        .DB     SaveMusic/256           ; f8
        .DB     LoadMusic/256           ; home

        .DB     EditInc/256             ; +
        .DB     EditDec/256             ; -

        .DB     LoopInc/256             ; arrow up

IncDecList:
        .DB     BLOCK,$01,$07           ;
        .DB     BLOCK,$ff,$07           ; 3
        .DB     SOUND,$01,$07           ; 6
        .DB     SOUND,$ff,$07           ; 9
        .DB     SPEED,$01,$0f           ; 12
        .DB     SPEED,$ff,$0f           ; 15
        .DB     CURSX,$01,$03           ; 18
        .DB     CURSX,$ff,$03           ; 21
        .DB     CURSY,$01,$0f           ; 24
        .DB     CURSY,$ff,$0f           ; 27

toptext:
        .DB     $02,$0c,$0f,$03,$0b,$2c,$30,$20
        .DB     $13,$0e,$04,$2c,$30,$20,$12,$30
        .DB     $20,$05,$0e,$16                 ; 1 byte truncated
        ;
LoadSaveMsg:
        .DB     $2c,$04,$01,$0f,$0c             ; ':daol'
        .DB     $2c,$05,$16,$01,$13             ; ':evas'

Hex:
        .DB     $30,$31,$32,$33,$34,$35,$36,$37
        .DB     $38,$39,$01,$02,$03,$04,$05,$06

hrowmap:
        .DB     $06,$06,$06,$06,$06,$06,$06,$08
        .DB     $06,$06,$06,$06,$06,$06,$06,$06
        .DB     $08,$06,$06                     ; 3 bytes truncated
        ;
lrowmap:
        .DB     $06,$06,$06,vuc,$06,$06,$06,$06
        .DB     vuc,$06,$06,$06,$06,vuc,$06,$06
        .DB     $06,$06,vuc,$06,$06,$06

midtext:
        .DB     $06,$09,$13,$09,$03,$08,$05,$0c
        .DB     $0c,$01,$20,$16,$31,$2e,$31,$20
        .DB     $20,$13,$10,$04,$2c

NoteEnd:
        .DB     $05,$0e,$04                     ; 'end'

VUchars:
        .DB     27,28,28,31,33,34,36,37,38

CurCols:
        .DB     $02,$0a,$13,$15,$04,$09,$0e,$13

CurSizes:
        .DB     $05,$06,$01,$01,$03,$03,$03,$03

CursorFlash:
        .DB     $03,$03,$03,$06,$00,$00,$00,$06

RowsL:
        .DEFINE raddr Screen+154
        .REPT 16
        .DB raddr&255
        .REDEFINE raddr raddr+22
        .ENDR
BotRowsL:
        .REDEFINE raddr Screen+22
        .REPT 4
        .DB raddr&255
        .REDEFINE raddr raddr+22
        .ENDR

RowsH:
        .REDEFINE raddr Screen+154
        .REPT 16
        .DB raddr/256
        .REDEFINE raddr raddr+22
        .ENDR
BotRowsH:
        .REDEFINE raddr Screen+22
        .REPT 4
        .DB raddr/256
        .REDEFINE raddr raddr+22
        .ENDR
.ENDS

.ORG $1a00
.SECTION "Music" FORCE

MusicData:
        .INCBIN "empty.bin"                     ; music routine & empty song
.ENDS

.ORG $1e00
.SECTION "Tables" FORCE

TapeBuffImage:
        .INCBIN "edtapeb.bin"                   ; VU meter data

NoteTable:
        .INCBIN "ednotes.bin"                   ; note names
.ENDS
