;       Compile with WLA-6510 (http://www.hut.fi/~vhelin/wla.html)

.INCLUDE "vic.inc"              ; memory map defines for WLA

*
*
*       FISICHELLA - Music Player for Commodore VIC-20 - v1.0
*
*       (c)2001 Aleksi Eeben (aleksi@cncd.fi)
*       http://www.cncd.fi/aeeben
*
*
*       jsr     $1c00   Initialize music
*       jsr     $1c03   Sound off
*       jsr     $1c06   Call once a frame to keep the music playing
*
*
*       Music data is located at $1a00-$1bbf. Player uses $1bc0-$1dff.
*
*

.DEFINE SeqData         $1a00   ; 32 steps x $04 bytes
.DEFINE BlockData       $1a80   ; 8 blocks x $10 bytes (fixed low byte $80)
.DEFINE SoundData       $1b00   ; 8 sounds x $10 bytes (fixed low byte $00)
.DEFINE SndSpeed        $1b80   ; 8 sounds x $01 byte
.DEFINE SndEnvTrig      $1b88   ; 8 sounds x $01 byte
.DEFINE EnvData         $1b80   ; 4 envs x $10 bytes (env 0 not used)
.DEFINE FreqData        $1bc0   ; note frequency table
.DEFINE Varis           $03ea   ; $16 bytes for player variables


.ENUM Varis
SequencerPos    DB      ; Sequencer position (n*$04) $ff=music ended
BlockStep       DB      ; Block step ($0-$f)
SpeedCount      DB      ; Block step time countdown
SongSpeed       DB      ; Change this to adjust song tempo
EnvCount        DB      ; Envelope step time countdown
EnvelopePos     DB      ; Envelope position, $00=no env in use

V1BlockTrans    DB      ; bits 7-4: block number, bits 3-0: transpose
V1Note          DB      ; Current note trigged from block data
V1Count         DB      ; Sound step time countdown
V1SoundPos      DB      ; Sound data position, $ff=voice off

V2BlockTrans    DB      ; same for the rest of voices
V2Note          DB
V2Count         DB
V2SoundPos      DB

V3BlockTrans    DB
V3Note          DB
V3Count         DB
V3SoundPos      DB

V4BlockTrans    DB
V4Note          DB
V4Count         DB
V4SoundPos      DB
.ENDE


.ORG $0
.SECTION "BANKHEADER"
        .dw     FreqData        ; CBM load address
.ENDS


.ORG FreqData
.SECTION "FREQTABLE" FORCE

FreqTable:
        .db     $82,$89,$90,$96,$9c,$a1,$a7,$ac,$b0,$b5,$b9,$bd
        .db     $82,$89,$90,$96,$9c,$a1,$a7,$ac,$b0,$b5,$b9,$bd
        .db     $c1,$c4,$c7,$ca,$cd,$d0,$d3,$d5,$d8,$da,$dc,$de
        .db     $e0,$e2,$e3,$e5,$e6,$e8,$e9,$ea,$eb,$ec,$ed,$ee
        .db     $ef,$f0,$f1,$f2,$f3,$f4,$f5,$f6,$f7,$f8,$f9,$fa
        .db     $fb,$fc,$fd,$fe
.ENDS


.ORG $1c00
.SECTION "MAIN" FORCE
        jmp     PlayerInit
        jmp     PlayerOff

PlayerFrame:
        dec     SpeedCount      ; next step in note data?
        beq     ReadNotes

UpdateVoices:
        lda     #$0a            ; first voice register $900a
        sta     $fd

        ldx     #$00
        jsr     UpdateOne
        inc     $fd
        ldx     #$04
        jsr     UpdateOne
        inc     $fd
        ldx     #$08
        jsr     UpdateOne
        inc     $fd
        ldx     #$0c
        jsr     UpdateOne

        lda     EnvelopePos     ; envelope in use?
        bne     CheckEnv

        rts


ReadNotes:
        lda     SongSpeed
        sta     SpeedCount

        lda     SequencerPos    ; song ended?
        cmp     #$ff
        beq     UpdateVoices

        ldx     #$00
        jsr     ReadOne
        ldx     #$04
        jsr     ReadOne
        ldx     #$08
        jsr     ReadOne
        ldx     #$0c
        jsr     ReadOne

        inc     BlockStep
        lda     BlockStep
        and     #$0f
        bne     UpdateVoices

        sta     BlockStep

        lda     SequencerPos
        clc
        adc     #$04
        sta     SequencerPos

readblocks:
        tay

        lda     SeqData,y
        cmp     #$fe
        beq     seqlooporend

        sta     V1BlockTrans
        jsr     initchentry     ; read block number for voices 2-4

        jmp     UpdateVoices

seqlooporend:
        lda     SeqData+1,y     ; $fe $ff = end, $fe $nn = loop
        sta     SequencerPos
        cmp     #$ff

        bne     readblocks
        jmp     UpdateVoices


CheckEnv:
        dec     EnvCount
        beq     envupdate
        rts

envupdate:
        lda     #EnvData&255
        sta     $fb
        lda     #EnvData/256
        sta     $fc

        ldy     EnvelopePos

afterloopcmd:
        lda     ($fb),y         ; get volume level from envelope
        ror
        ror
        ror
        ror
        and     #$0f
        sta     $fd

        lda     ($fb),y
        and     #$0f
        beq     envcommand
        sta     EnvCount        ; set env step time from envelope

        lda     $900e           ; change volume level
        and     #$f0
        ora     $fd             ; without changing the auxiliary
        sta     $900e           ; color in the same register

        iny
        sty     EnvelopePos

        rts

envcommand:
        lda     $fd             ; get command param (hi nybble)
        beq     envendcmd       ; $00 = end of envelope

        lda     EnvelopePos     ; $n0 = loop at position n
        and     #$f0
        ora     $fd
        sta     EnvelopePos

        tay
        jmp     afterloopcmd

envendcmd:
        sta     EnvelopePos
        rts


ReadOne:
        ldy     BlockStep

        lda     V1BlockTrans,x  ; get block number
        bmi     noblock
        sta     $fd
        and     #$70
        ora     #$80            ; address low byte: $80 + block * $10
        sta     $fb
        lda     #BlockData/256
        sta     $fc

        lda     ($fb),y
        bne     newnote
noblock:
        rts
newnote:
        and     #$1f
        sta     $fe

        lda     $fd             ; get transpose from low 4 bits in $fd
        and     #$0f
        sec
        sbc     #$08            ; actual transpose value = $fd - 8
        clc
        adc     $fe             ; add note value
        sta     V1Note,X        ; set channel note

        lda     #$01
        sta     V1Count,x       ; trig sound at next voice update

        lda     ($fb),y         ; set sound position
        ror
        and     #$70
        sta     V1SoundPos,x

        ror
        ror
        ror
        ror
        and     #$07
        tay
        lda     SndEnvTrig,y    ; env trig table
        bne     envtrig
        rts
envtrig:
        sta     EnvelopePos
        lda     #$01
        sta     EnvCount

        rts


UpdateOne:
        lda     V1SoundPos,x
        cmp     #$ff
        beq     voiceoff
        dec     V1Count,x
        beq     advancesnd
        rts
voiceend:
        ldy     $fd
        sta     $9000,y
        lda     #$ff
        sta     V1SoundPos,x
voiceoff:
        rts

advancesnd:
        lda     V1SoundPos,x
        sta     $fb
        ror
        ror
        ror
        ror
        and     #$0f
        tay
        lda     SndSpeed,y      ; get sound speed from table
        sta     V1Count,x

afterloop:
        lda     #SoundData/256
        sta     $fc
        ldy     #$00
        lda     ($fb),y         ; get sound step
        beq     voiceend

        cmp     #$10
        bcc     voiceloop

        and     #$1f
        clc
        adc     #$10
        sta     $fe
        lda     V1Note,x
        clc
        adc     $fe

        tay
        cpy     #$40
        bcc     _nohiwrap
        ldy     #$3f
_nohiwrap:
        cpy     #$c0
        bcc     _nolowrap
        ldy     #$00
_nolowrap:

        lda     FreqTable,y     ; get note frequency from freq table
        sta     $fe

        ldy     #$00
        lda     ($fb),y         ; add vibrato phase value
        rol
        rol
        rol
        rol
        and     #$07
        sec
        sbc     #$04
        clc
        adc     $fe

        ldy     $fd
        sta     $9000,y         ; play note

        inc     V1SoundPos,x
        rts

voiceloop:
        and     #$0f
        sta     $fe
        lda     $fb
        and     #$f0
        ora     $fe
        sta     $fb
        sta     V1SoundPos,x

        jmp     afterloop


PlayerInit:
        jsr     PlayerOff

        lda     $1dff           ; song speed from last byte
        sta     SongSpeed
        lda     #$01            ; read notes next frame
        sta     SpeedCount

        lda     #$ff
        sta     V1SoundPos
        sta     V2SoundPos
        sta     V3SoundPos
        sta     V4SoundPos

        lda     $900e           ; initialize volume
        and     #$f0
        ora     #$08            ; without changing the auxiliary
        sta     $900e           ; color in the same register

        lda     SeqData         ; prepare first sequencer step
        sta     V1BlockTrans
        ldy     #$00

initchentry:
        lda     SeqData+1,y     ; also called from the frame routine
        sta     V2BlockTrans    ; to save a couple of bytes
        lda     SeqData+2,y
        sta     V3BlockTrans
        lda     SeqData+3,y
        sta     V4BlockTrans

        rts


PlayerOff:
        lda     #$00
        ldy     #$15
_clr:
        sta     Varis,y         ; clear variable area
        cpy     #$04
        bcs     _novoice
        sta     $900a,y         ; all voices off
_novoice:
        dey
        bpl     _clr

        rts
.ENDS

.ORG $1df0
.SECTION "TAG" FORCE
        .DB     "FISICHELLAV1.0"
.ENDS

.ORG $1dff
.SECTION "SPEED" FORCE
        .DB     6               ; initial song speed
.ENDS
